/*
 
File:	test_gfx2d.c

Purpose:	Test and Demo code for 2D graphics engine.

*/


#include "xparameters.h"	// generated system parameters
#include "xbasic_types.h"	// Xilinx basic types for device drivers
#include "xio.h"			// For register reads/writes via OPB bus

#include "gfx2d.h"			// API for gfx2d operations

#include "pic.h"			// bitmap data (picture of group members)

// Function prototypes
void test_ops ();
void demo ();
void drawSnake (Xuint32 x, Xuint32 y, Xuint32 width, Xuint32 height, Xuint32 RGB);

int main() {

	// Test all operations
	// test_ops();

	// Demo code
	demo();
}

// Tests all basic ops of graphics engine
void test_ops () {

	Xuint32 x, y, w, h;
	char msg [] = "Test";

	clearScreen(BLACK);

	fillRect(200,300,100,150,CYAN);

	drawRect(250,350,50,80,BLUE); 

	drawSquare(150,150,100,RED);  
	drawSquare(160,160,100,BLUE);
	drawSquare(170,170,100,WHITE);

	fillSquare(400,100,50,GREEN);

	// Case 1: not steep, x1 > x0, slope -ve
	lineOp(100,200,300,300,VIOLET);

	// Case 2: not steep, x0 > x1, slope -ve
	lineOp(300,305,100,205,YELLOW);

	// Case 3: steep, x1 > x0, slope -ve
	lineOp(300,200,350,450,WHITE);

	// Case 4: steep, x0 > x1, slope -ve
	lineOp(355,455,305,205,CYAN);

	// Case 5: not steep, x1 > x0, slope +ve
	lineOp(100,200,300,100,VIOLET);

	// Case 6: not steep, x0 > x1, slope +ve
	lineOp(300,105,100,205,YELLOW);

	// Case 7: steep, x1 > x0, slope +ve
	lineOp(300,200,350,150,WHITE);

	// Case 8: steep, x0 > x1, slope +ve
	lineOp(355,155,305,205,CYAN);

	drawTriangle(400,400,400,450,550,420,WHITE);
	drawTriangle(417,383,517,383,550,420,RED);
	drawTriangle(422,388,522,385,555,425,WHITE);
	drawTriangle(427,393,527,395,560,430,VIOLET);
	
	drawRect(305,350,75,75,GREEN); 
	drawRect(315,360,75,75,BLUE); 
	drawRect(320,370,75,75,WHITE);

	charOp (200, 200, 'D', WHITE);
	charOp (207, 200, 'e', CYAN);
	charOp (214, 200, 'M', YELLOW);
	charOp (221, 200, 'o', BLUE);

	pixelOp (240, 200, GREEN);
	pixelOp (242, 200, BLUE);
	pixelOp (244, 200, WHITE);
	pixelOp (240, 203, RED);
	pixelOp (242, 203, CYAN);
	pixelOp (244, 203, YELLOW);

	drawString (250, 200, GREEN, msg);

	x = 300;
	y = 50;
	drawString (x, y, BLUE, "GFX");
	drawString (x+30, y, BLUE, "ENGINE");

	// let's try some animation ...
	
	x = 300;
	y = 300;
	w = 50;
	h = 70;

	// rectangle (filled) moving right
	while (x < 500) {

		// clear previous rectangle
		fillRect(x-2,y,1,h,BLACK);

		fillRect(x,y,w,h,GREEN);
		x = x + 1;

		// wait a bit before drawing next frame
		delay(2);
	}

	x = 300;
	y = 200;
	w = 80;
	h = 20;

	// rectangle (hollow) moving down
	while (y < 400) {

		// clear previous rectangle
		fillRect(x,y-2,w,1,BLACK);
		fillRect(x+1,y+h-2,w-1,1,BLACK);

		drawRect(x,y,w,h,BLUE);
		y = y + 1;

		// wait a bit before drawing next frame
		delay(2);
	}
}

// Demo code used during presentation
void demo() {

	Xuint32 x, y, w, h;
	int i;

	for (i = 0; i < 10; i++) clearScreen(BLACK);

	x = 300;
	y = 50;
	w = 90;
	h = 27;

	drawRect(x - 10,y - 10,w,h,BLUE);

	drawString (x, y, BLUE, "GFX");
	drawString (x+30, y, BLUE, "ENGINE");

	// rectangle (filled) moving up on left side of screen
	x = 35;
	y = 480;
	w = 25;
	h = 25;

	while (y > h) {

		fillRect(x,y-h,w,h,CYAN);
		fillRect(640-w,y-h,w,h,CYAN);
		y--;
	}

	// rectangle (filled) moving right at top of screen
	x = 0;
	y = 0;
	w = 25;
	h = 25;

	while (x <= 615) {

		fillRect(x,y,w,h,CYAN);
		fillRect(615-x,480-y-h,w,h,CYAN);
		x++;
	}

	// draw a star
	drawStar (350, 325, 5, 50, GREEN);

	// rotate a square with clear = 0
	for (i = 0; i < 4; i++) {
		rotateSquare90(150, 100, 25, RED, 0);
	}

	// rotate a square with clear = 1
	for (i = 0; i < 8; i++) {
		rotateSquare90(150, 250, 25, RED, 1);
	}

	delay(4);

	// rotate a star with clear=0
	for (i = 0; i < 4; i++) {
		rotateStar90 (500, 300, 5, 40, VIOLET, 0);
	}

	delay(4);

	// rotate a star with clear=1
	for (i = 0; i < 4; i++) {
		rotateStar90 (500, 100, 5, 40, VIOLET, 1);
	}

	delay(4);

	// display ppm to screen
	ppmOp(275,125,pic);
	
	delay(4);

	x = 100;
	y = 400;
	w = 100;
	h = 24;

	//drawSnake (x, y, w, h, GREEN);
	while (x <= 450) {

		// clear previous snake
		fillRect(x-2,y,1,h,BLACK);

		drawSnake (x, y, w, h, GREEN);
		x++;
	}

}

// Demos ability to draw squares, triangles
void drawSnake (Xuint32 x, Xuint32 y, Xuint32 width, Xuint32 height, Xuint32 RGB) {

	Xuint32 mid = height/2;
	Xuint32 mid2 = height/4;

	// Mouth
	drawTriangle(x+width, y, x+width, y+mid, x+width+height, y+mid2,RGB);
	drawTriangle(x+width, y+height, x+width, y+mid, x+width+height, y+mid+mid2,RGB);
	
	// Eye
	//drawSquare (x+width+5, y+mid2, 3, RGB);

	// Body
	fillRect (x, y, width, height, RGB);

	// Spots on body
	//pixelOp (x-width+(Xuint32)(width/6), y+(Xuint32)height/4, BLUE);
	//pixelOp (x-width+(Xuint32)(2*width/6), y-(Xuint32)height/5, WHITE);
	//pixelOp (x-width+(Xuint32)(3*width/6), y+(Xuint32)height/3, RED);
	//pixelOp (x-width+(Xuint32)(4*width/6), y-(Xuint32)height/4, VIOLET);
	//pixelOp (x-width+(Xuint32)(5*width/6), y+(Xuint32)height/5, YELLOW);
}
