-- clk_align.vhd file
----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_Logic_1164.all; 
use IEEE.STD_Logic_arith.all;
use IEEE.STD_Logic_unsigned.all;


entity clk_align is
  port(
    external_clk      : in  STD_LOGIC;
    extend_dcm_reset  : in  STD_LOGIC;
    dcm0_locked       : in  STD_LOGIC;
    dcm1_locked       : in  STD_LOGIC;
    fpga_reset        : out STD_LOGIC;
    dcm_reset         : out STD_LOGIC
  );
end entity clk_align;

architecture behaviour of clk_align is
  signal dcm_reset_internal   : STD_LOGIC;
  signal fpga_reset_internal  : STD_LOGIC;
  signal enable_reset_counter : STD_LOGIC;
  signal dcm_reset_counter    : STD_LOGIC_VECTOR(0 to 15) := X"0000";
begin
  -- Assume active-high DCM and system reset signals
  dcm_reset_internal    <=  (not dcm_reset_counter(0));
  fpga_reset_internal   <=  (not dcm0_locked) or (not dcm1_locked);

  -- Keep counting until both the DCM and the FPGA are out of reset
  enable_reset_counter  <=  dcm_reset_internal or fpga_reset_internal;

  -- Relay the internal signals to the output pins
  dcm_reset             <=  dcm_reset_internal;
  fpga_reset            <=  fpga_reset_internal;

  dcm_reset_counter_inst : process (external_clk)
  begin
    if(external_clk = '1' and external_clk'event) then
      if(extend_dcm_reset = '1') then
        dcm_reset_counter <= X"0000";
      elsif(enable_reset_counter = '1') then
        dcm_reset_counter <=  dcm_reset_counter + 1;
      end if;
    end if;
  end process;

end architecture behaviour;

