`timescale 1ns/1ps

/*

File: Blit.v

Purpose: Fill rectangle with starting coordinate at x0,y0 and ending coordinate x1,y1 with colour RGB

Author: Hafiz Noordin

*/

module Blit (
	iCLK,
	iRST,
	iSOFT_RST,

	// Decoder interface
	iREQ,
	iDATA,
	oRTR,

	// output FIFO interface
	iFIFO_FULL,
	oFIFO_WE,
	oFIFO_WD
);

input			iCLK;
input			iRST;
input			iSOFT_RST;
input			iREQ;
input [159:0]	iDATA;
output			oRTR;

input			iFIFO_FULL;
output			oFIFO_WE;
output [43:0]	oFIFO_WD;

reg [9:0]		x0, x1, y0, y1;					// store input data
reg [23:0]		RGB;

wire [9:0]		x0_in, x1_in, y0_in, y1_in;
wire [23:0]		RGB_in;

reg				busy;
reg [9:0]		x;								// x,y counters
reg [9:0]		y;

assign RGB_in = iDATA[23:0];
assign x0_in = iDATA[41:32];
assign y0_in = iDATA[73:64];
assign x1_in = iDATA[105:96];
assign y1_in = iDATA[137:128];

assign oRTR = !busy;							// ready to receive = !busy

assign oFIFO_WE = busy && !iFIFO_FULL;			// write enable/data
assign oFIFO_WD = {x, y, RGB};

always @(posedge iCLK) begin
	if (iRST || iSOFT_RST) begin
		busy <= 0;
		x <= 0;
		y <= 0;
		RGB <= 0;
	end
	
	// Wait for valid request (iREQ == 1, busy == 0)
	else if (iREQ && !busy) begin
		busy <= 1;

		// latch input data
		RGB <= RGB_in;
		x0 <= x0_in;
		y0 <= y0_in;
		x1 <= x1_in;
		y1 <= y1_in;
		x <= x0_in;
		y <= y0_in;
	end

	// Write pixel for each line y0 <= y <= y1 such that x0 <= x <= x1
	// - stall if fifo full
	else if (busy && !iFIFO_FULL) begin

		// Next pixel in current line
		if (x < x1) begin
			x <= x + 1;
		end

		// Start next line
		else if (y < y1) begin
			x <= x0;
			y <= y + 1;
		end

		// Done when y==y1 and x > x1
		else begin
			busy <= 0;
		end
	end
end

endmodule
