`timescale 1ns / 1ps

//////////////////////////////////////////////////////////////////////////////////
// Company: UFT
// Engineer: 	MOHAMAD AYAD
// 
// Create Date:    18:33:56 03/07/2007 
// Design Name:   Char display
// Module Name:    Char 
// Project Name:  2D ENGINE
// Target Devices: 	 Virtex 2
// Tool versions: 	8.2
// Description: display characters on the VGA display. outputs address and RGB of pixel
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module Char(
	clk, 
	reset,
	soft_reset, 
	
	req, 
	rtr,
	data, 
	
	fifo_full, 
	fifo_we, 
	fifo_wd
);
	
	parameter width_in = 160;		//width of in data
	parameter width_out = 44;		//width of out data
	
	input clk;
	input reset,soft_reset;			//Active H, synchronous resets
	
	//communication with Decoder
	input req;						//active H, validates input data
	output rtr;						//active H, Char is ready to read in data (not busy)
	input [width_in-1:0] data;		//input command data from Decoder
	reg [50:0] data_reg;			//store input command data,eliminates zeros
	
	//communication with Char FIFO
	input fifo_full;				//active H, char operation fifo is full
	output fifo_we;					//active H, output data from char is valid
	output [width_out-1:0] fifo_wd;	//output data from char to fifo

	// store command values
	wire [23:0] RGB = data_reg[23:0];
	wire [9:0] x0 = data_reg[33:24];	//initial x coordinates
	wire [9:0] y0 = data_reg[43:34];	//initial y coordinates
	wire [6:0] ascii = data_reg[50:44];	//ASCII value
	wire [5:0] addr = ascii[5:0];		//address of ASCII in BRAM
 
	reg [9:0] x,y;					//current x and y coordinates
	reg [2:0] countx,county;		//x and y counters
	wire [63:0] char;				//Character info from BRAM
	reg [63:0] char_shift;			//stores char info from BRAM

	//control signals
	reg rtr;
	reg Eaddr;		//active H, enables BRAM
	reg fifo_we;	
	
	//SM variables
	reg [2:0] ps,ns;
	parameter [2:0] S1=001,	//wait for valid data
						 S3=011,	//read ASCII char value
						 S2=010,	//store values
						 S6=110,	//check MSB bit
						 S4=100;	//Left shift char value
	
	//Control Path
	always @ (ps,req, fifo_full, char_shift) begin
		rtr=0;Eaddr=0;fifo_we=0;ns = ps; //defaults
		case(ps)
			S1: begin
					if(req)	ns=S3;
					else	ns=S1;
					rtr=1;
				 end
			S3: begin
					Eaddr=1;
					ns=S2;
				 end
			S2: ns=S6;
			S6: if(char_shift==0)	ns=S1;
				 else 				ns=S4;
			S4: if(!fifo_full) begin
					ns=S6;
					fifo_we=char_shift[63];
				 end
				 else ns=S4;
			default: ns = 3'bxxx;
		endcase
	end

	//Data Path
	assign fifo_wd = {x,y,RGB};
	always @(posedge clk) begin
		if(reset || soft_reset) begin
			ps <= S1;
			data_reg <= 0;
			x <= 0;
			y <= 0;
			countx <= 0;
			county <= 0;
			char_shift <= 0;
		end
		else begin
			ps <= ns;
			case(ps)
				S1:	begin 
						if(req) begin
							data_reg[23:0]  <= data[23:0];
							data_reg[33:24] <= data[41:32];
							data_reg[43:34] <= data[73:64];
							data_reg[50:44] <= data[102:96];
							countx <= 0;
							county <= 0; 
						end
					end
				S2:	begin
						char_shift <= char;
						x <= x0;
						y <= y0;
					end
				S6:	begin 
						if(char_shift[63]) begin
							x <= x0 + countx;
							y <= y0 + county;
						end

						if (countx == 3'b111) begin
							county <= county + 1;
							countx <= 0;
						end
						else begin
							countx <= countx + 1;
						end
					end
				S4:	begin
						if (!fifo_full) begin
							char_shift <= char_shift << 1;
						end
					end 
			endcase
		end
	end

//Memory
brom CharBRAM(
	.addr(addr),
	.clk(clk),
	.dout(char),
	.en(Eaddr));

endmodule
