`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: UFT
// Engineer: Mohamad ayad
// 
// Create Date:    10:14:38 02/20/2007 
// Design Name: Decoder
// Module Name:    decoder 
// Project Name: 2D Engine
// Target Devices: Virtex 2
// Tool versions: ISE 8.2
// Description: store opb data in a fifo, and use a decoder to decode data
//					 for the algorithms.
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module Decoder(
	clk,
	reset,
	soft_reset,
	
	op,
	data,
	enq,
	full,
	
	busy_line,
	busy_blit,
	busy_char,
	busy_pixel,
	
	valid_line,
	valid_blit,
	valid_char,
	valid_pixel,
	
	data_out
);

	parameter data_width = 160;		// width of data from OPB bus
	parameter op_width = 3;			// width of op from OPB bus

	// 2D ops
	parameter OP_LINE = 0;
	parameter OP_BLIT = 1;
	parameter OP_CHAR = 2;
	parameter OP_PIXEL= 3;
	parameter OP_DBG  = 4;
	 
	input clk;
	input reset;				//active H synchronous system Reset
	input soft_reset;			//active H synchronous soft Reset
	input enq;					//active H asserted to add data to fifo
	
	input [op_width-1:0] op;	//op code of the command
	input [data_width-1:0]data;	//data from OPB bus
	input busy_line;			//active H line operation busy
	input busy_blit;			//active H blit operation busy
	input busy_char;			//active H Char operation busy
	input busy_pixel;			//active H Pixel operation busy
	
	output full;				//active H if fifo is full to OBP interface
	output valid_line;			//active H ouput data is valid for line
	output valid_blit;			//active H ouput data is valid for blit
	output valid_char;			//active H ouput data is valid for char
	output valid_pixel;			//active H ouput data is valid for pixel
	output [data_width-1:0] data_out;	//data to operations depends on validity
	
	wire busy = (busy_line || busy_blit || 
						busy_char || busy_pixel);	//active H OPERATIONS is busy
	
	wire deq;		//active H dequeue of FIFO
	
	//wrapped FIFOs outputs
	wire fifo_valid;				//active H fifo ouput validity
	wire fifo_empty;				//active H fifo is empty
	wire [op_width-1:0] fifo_op;
	wire [data_width-1:0] fifo_data;
		
	//Command fifo to store data from OPB and output to decoder
	Fifo_2 #(data_width+op_width, 16, 4) Command_fifo(
		.iCLK(clk),
		.iRST_CLK(reset || soft_reset),
		.iENQUEUE(enq),
		.iDATAIN({op,data}),
		.oFULL(full),
		.iDEQUEUE(deq),
		.oDATAOUT({fifo_op,fifo_data}),
		.oEMPTY(fifo_empty)
	);
	
	//Digital circuit of Decoder, validates operations dependin on op
	assign fifo_valid = !fifo_empty;
	assign deq = !busy && fifo_valid;
	assign valid_line = fifo_valid && fifo_op == OP_LINE;
	assign valid_blit = fifo_valid && fifo_op == OP_BLIT;
	assign valid_char = fifo_valid && fifo_op == OP_CHAR;
	assign valid_pixel= fifo_valid && fifo_op == OP_PIXEL;
	assign data_out = fifo_data;

endmodule
