`timescale 1ns/1ps
/*

File: Display.v

Purpose: Experimental module (NOT USED IN CURRENT DESIGN) to fill entire screen with single colour.

Author: Hafiz Noordin

*/

module Display (
	pixel_clock,
	reset,
	user_vga_addr,
	user_vga_read,
	user_vga_write,
	user_vga_write_byte_a,
	user_vga_write_byte_b,
	user_vga_write_byte_c,
	user_vga_write_byte_d,
	user_vga_enable,
	user_vga_read_data,
	user_vga_write_data,
	user_access_ok,
	user_clock_enable,
	user_load_vga_addr,
	iRGB_DATA
);

// Display interface
input						pixel_clock		/* synthesis syn_noclockbuf = 1 */; // pixel clock input
input						reset;			// reset
output [18:0]				user_vga_addr;
output						user_vga_read;
output						user_vga_write;
output						user_vga_write_byte_a;
output						user_vga_write_byte_b;
output						user_vga_write_byte_c;
output						user_vga_write_byte_d;
output						user_vga_enable;
input  [31:0]				user_vga_read_data;
output [31:0]				user_vga_write_data;
input						user_access_ok;
output						user_clock_enable;
output						user_load_vga_addr;	// flag to load the address specified by the user_vga_add bus 

input [23:0]				iRGB_DATA;

reg [18:0]					user_vga_addr;
reg							user_vga_read;
reg							user_vga_write;
reg							user_vga_write_byte_a;
reg							user_vga_write_byte_b;
reg							user_vga_write_byte_c;
reg							user_vga_write_byte_d;
reg							user_vga_enable;
reg [31:0]					user_vga_write_data;
reg							user_clock_enable;
reg							user_load_vga_addr;

reg [18:0]					address_count;

// Drive display / framebuffer
always @(posedge pixel_clock) begin
	
	if (user_access_ok) begin
		// Increment address while user_access_ok is HIGH
		user_vga_addr <= address_count;
		address_count <= address_count + 1;

		// Memory control signals
		user_vga_read <= 1'b0;
		user_vga_write <= 1'b1;
		user_vga_write_byte_a <= 1'b1;
		user_vga_write_byte_b <= 1'b1;
		user_vga_write_byte_c <= 1'b1;
		user_vga_write_byte_d <= 1'b0;
		user_vga_enable <= 1'b1;
		user_clock_enable <= 1'b1;
		user_load_vga_addr <= 1'b1;
	end

	// Reset address counter to zero once reaches 640x480
	if (address_count == 19'h4B000) begin
		address_count <= 19'h00000;
	end

	user_vga_write_data <= {8'b0, iRGB_DATA};
end

endmodule
