`timescale 1ns/1ps
/*

File: Fifo.v

Purpose: First fifo used in design (CURRENTLY NOT INSTANTIATED IN DESIGN).

*/

module Fifo (clock,reset,data_in,enq,full,data_out,valid_out,deq);

	parameter width = 44;
	parameter depth = 16;
	parameter addr = 4;	// log(length) base 2

	input clock, reset;

	input [width-1:0] data_in;
	input enq;
	output full;
	reg full;
	
	output [width-1:0] data_out;
	reg [width-1:0] data_out;
	output valid_out;
	reg valid_out;
	input deq;

	reg [width-1:0] fifo_data [0:depth-1];
	reg [addr:0] fifo_head, fifo_tail;
	reg [addr:0] next_tail;


// accept input
	wire next_full = fifo_head[addr-1:0] == next_tail[addr-1:0] &&
							fifo_head[addr] != next_tail[addr];
							
	wire is_full = fifo_head[addr-1:0] == fifo_tail[addr-1:0] &&
							fifo_head[addr] != fifo_tail[addr];

	//always @(posedge clock or negedge reset) begin
	always @(posedge clock) begin
		if (reset) begin
			fifo_tail <= 0;
			next_tail <= 1;
			full <= 0;
		end else begin
			if (!full && enq) begin
			// We can only enqueue when not full
				fifo_data[fifo_tail[addr-1:0]] <= data_in;
				next_tail <= next_tail + 1;
            	fifo_tail <= next_tail;
            
            	// We have to compute if it's full on next cycle
            	full <= next_full;
        	end else begin
           		full <= is_full;
        	end
    	end
	end


// provide output
	wire is_empty = (fifo_head == fifo_tail)?1:0;
//always @(posedge clock or negedge reset) begin
always @(posedge clock) begin
	if (reset) begin
		valid_out <= 0;
		data_out <= 0;
		fifo_head <= 0;
	end else begin
		// If no valid out or we're dequeueing, we want to grab
		// the next data.  If we're empty, we don't get valid_out,
		// so we don't care if it's garbage.
		if (!valid_out || deq) begin
			data_out <= fifo_data[fifo_head[addr-1:0]];
		end
		
		if (!is_empty) begin
			if (!valid_out || deq) begin
				fifo_head <= fifo_head + 1;
			end
			valid_out <= 1;
		end else begin
			if (deq) valid_out <= 0;
		end
	end
end


endmodule
