`timescale 1ns/1ps
/*

File: MemIF.v

Purpose: Experimental module (NOT USED IN CURRENT DESIGN) to read/write from ZBT memory

Author: Hafiz Noordin
*/

`define SIMPLE_WRITE

module MemIF (
	iCLK,
	iRST,
	iSOFT_RST,

	iVALID,
	iDATA,
	oRTR,

	user_access_ok,
	user_vga_read_data,

	user_clock_enable,
	user_load_vga_addr,
	user_vga_addr,
	user_vga_enable,
	user_vga_read,
	user_vga_write,
	user_vga_write_byte_a,
	user_vga_write_byte_b,
	user_vga_write_byte_c,
	user_vga_write_byte_d,
	user_vga_write_data
);

input			iCLK;
input			iRST;
input			iSOFT_RST;

input			iVALID;
input			iDATA;
output			oRTR;

input			user_access_ok;
input  [31:0]	user_vga_read_data;

output			user_clock_enable;
output			user_load_vga_addr;
output [18:0]	user_vga_addr;
output			user_vga_read;
output			user_vga_write;
output			user_vga_write_byte_a;
output			user_vga_write_byte_b;
output			user_vga_write_byte_c;
output			user_vga_write_byte_d;
output			user_vga_enable;
output [31:0]	user_vga_write_data;

`ifdef SIMPLE_WRITE

wire [9:0]					x;
wire [9:0]					y;
wire [23:0]					RGB;

assign {x,y,RGB} = iDATA;
//assign user_vga_addr = y * 640 + x;
assign user_vga_addr = {y, 9'b0} + {2'b0, y, 7'b0} + {9'b0, x}; // y * 640 + x
assign user_vga_read = 0;
assign user_vga_write_byte_d = 0;
assign {user_vga_write, user_vga_write_byte_a, user_vga_write_byte_b, user_vga_write_byte_c, user_vga_enable, user_clock_enable, user_load_vga_addr} = iVALID && user_access_ok ? {7{1'b1}} : 0;
assign user_vga_write_data = {8'b0, RGB};
assign oRTR = user_access_ok;

`else

parameter S0 = 0;
parameter S1 = 1;
parameter S2 = 2;
parameter S3 = 3;
parameter S4 = 4;
parameter S5 = 5;
parameter S6 = 6;

reg			user_clock_enable;
reg			user_load_vga_addr;
reg [18:0]	user_vga_addr;
reg			user_vga_read;
reg			user_vga_write;
reg			user_vga_write_byte_a;
reg			user_vga_write_byte_b;
reg			user_vga_write_byte_c;
reg			user_vga_write_byte_d;
reg			user_vga_enable;
reg [31:0]	user_vga_write_data;

reg [2:0]	state, state_next;

reg [9:0]	x;
reg [9:0]	y;
reg [23:0]	RGB;

always @(state or iVALID or user_access_ok or user_vga_read_data) begin
	state_next = state;

	case (state)
		S0:	begin
				if (iVALID) begin
					state_next = user_access_ok ? S2 : S1;
				end
			end

		S1: begin
				// Wait for user_access_ok
				state_next = user_access_ok ? S2 : S1;
			end

		S2: begin
				// Write state
				state_next = user_access_ok ? S4 : S3;
			end

		S3: begin
				// Wait for user_access_ok
				state_next = user_access_ok ? S4 : S3;
			end

		S4: begin
				// Read state 1
				state_next = S5;
			end

		S5: begin
				// Read state 2
				state_next = S6;
			end

		S6: begin
				// Check read data: if matches we're done, else write again
				state_next =	(user_vga_read_data[23:0] == RGB) ? S0 :
								user_access_ok ? S2 :
								S1;
			end

		default: state_next = 'bx;
	endcase
end

always @(posedge iCLK) begin
	if (iRST || iSOFT_RST) begin
		state <= S0;
		user_clock_enable <= 0;
		user_load_vga_addr <= 0;
		user_vga_addr <= 0;
		user_vga_read <= 0;
		user_vga_write <= 0;
		user_vga_write_byte_a <= 0;
		user_vga_write_byte_b <= 0;
		user_vga_write_byte_c <= 0;
		user_vga_write_byte_d <= 0;
		user_vga_enable <= 0;
		user_vga_write_data <= 0;
		{x,y,RGB} <= 0;
	end
	else begin
		state <= state_next;

		if (state == S0 && iVALID) begin
			{x,y,RGB} <= iDATA;
		end

		if (state_next == S2) begin
			// Write
			user_clock_enable <= 1;
			user_load_vga_addr <= 1;
			user_vga_addr <= y * 640 + x;
			user_vga_read <= 0;
			user_vga_write <= 1;
			user_vga_write_byte_a <= 1;
			user_vga_write_byte_b <= 1;
			user_vga_write_byte_c <= 1;
			user_vga_write_byte_d <= 0;
			user_vga_enable <= 1;
			user_vga_write_data <= {8'b0, RGB};
		end
		else if (state_next == S4) begin
			// Read
			user_clock_enable <= 1;
			user_load_vga_addr <= 1;
			user_vga_addr <= y * 640 + x;
			user_vga_read <= 1;
			user_vga_write <= 0;
			user_vga_write_byte_a <= 0;
			user_vga_write_byte_b <= 0;
			user_vga_write_byte_c <= 0;
			user_vga_write_byte_d <= 0;
			user_vga_enable <= 1;
			user_vga_write_data <= 0;
		end
		else begin
			user_clock_enable <= 0;
			user_load_vga_addr <= 0;
			user_vga_addr <= 0;
			user_vga_read <= 0;
			user_vga_write <= 0;
			user_vga_write_byte_a <= 0;
			user_vga_write_byte_b <= 0;
			user_vga_write_byte_c <= 0;
			user_vga_write_byte_d <= 0;
			user_vga_enable <= 0;
			user_vga_write_data <= 0;
		end
	end
end

assign oRTR = state == S0;
	
`endif

endmodule


