`timescale 1ns/1ps

module blit_tb ();

reg			OPB_Clk;
reg			OPB_Rst;
reg 		soft_reset;
reg 		req_blit;
reg	[9:0]	x0, x1, y0, y1;
reg [23:0]	RGB;
wire		blit_rtr;

wire [127:0]	input_data = {44'b0, y1, x1, 12'b0, y0, x0, 8'b0, RGB};

reg 		user_access_ok;
wire [18:0]	user_vga_addr;
wire 		user_vga_read;
wire 		user_vga_write;
wire 		user_vga_write_byte_a;
wire 		user_vga_write_byte_b;
wire 		user_vga_write_byte_c;
wire 		user_vga_write_byte_d;
wire 		user_vga_enable;
wire [31:0]	user_vga_write_data;
wire 		user_clock_enable;
wire 		user_load_vga_addr;

// Design Under Test
Blit ublit(
	.iCLK(OPB_Clk),
	.iRST(OPB_Rst),
	.iSOFT_RST(soft_reset),
	.iDATA(input_data),
	.iREQ(req_blit),
	.oRTR(blit_rtr),

	.iVGA_ACCESS_OK(user_access_ok),
	.oVGA_ADDR(user_vga_addr),
	.oVGA_READ(user_vga_read),
	.oVGA_WRITE(user_vga_write),
	.oVGA_WRITE_BYTE_A(user_vga_write_byte_a),
	.oVGA_WRITE_BYTE_B(user_vga_write_byte_b),
	.oVGA_WRITE_BYTE_C(user_vga_write_byte_c),
	.oVGA_WRITE_BYTE_D(user_vga_write_byte_d),
	.oVGA_ENABLE(user_vga_enable),
	.oVGA_WRITE_DATA(user_vga_write_data),
	.oCLOCK_ENABLE(user_clock_enable),
	.oLOAD_VGA_ADDR(user_load_vga_addr)
);


// Clock gen
always #10 OPB_Clk = ~OPB_Clk;

// Stimulate inputs
initial begin
	OPB_Clk = 0;
	OPB_Rst = 0;

	x0 = 0;
	x1 = 0;
	y0 = 0;
	y1 = 0;
	RGB = 0;
	req_blit = 0;
	user_access_ok = 1;

	reset;

	doBlit(0, 0, 10, 2, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);

	reset;

	//doBlit(500, 200, 510, 205, 24'hFF_0000);
	//repeat(5) @(posedge OPB_Clk);

	$stop;
end

initial #1000 $stop;

// Asserts reset for 2 clocks then deasserts
task reset;
	begin
		#3 OPB_Rst = 1;
		repeat(2) @(posedge OPB_Clk);
		#3 OPB_Rst = 0;
	end
endtask

task doBlit;
	input [9:0] x0_in;
	input [9:0] y0_in;
	input [9:0] x1_in;
	input [9:0] y1_in;
	input [23:0] RGB_in;
	begin
		#3 x0 = x0_in;
		y0 = y0_in;
		x1 = x1_in;
		y1 = y1_in;
		RGB = RGB_in;
		req_blit = 1;
		@(posedge OPB_Clk);
		#3 req_blit = 0;
		@(posedge blit_rtr);
	end
endtask

endmodule
