`timescale 1ns/1ps

module gfx2d_tb ();

parameter C_BASEADDR = 32'h80040000;
parameter C_HIGHADDR = 32'h800400ff; // 256 byte space
parameter C_OPB_AWIDTH = 32;
parameter C_OPB_DWIDTH = 32;

reg							OPB_Clk;
reg							OPB_Rst;
reg [C_OPB_AWIDTH-1:0]		OPB_ABus;
reg [C_OPB_DWIDTH-1:0]		OPB_DBus;
reg [C_OPB_DWIDTH/8-1:0]	OPB_BE;
reg							OPB_RNW;
reg							OPB_select;
reg							OPB_seqAddr;
reg							OPB_hwXfer;
reg							OPB_fwXfer;
wire						Sln_xferAck;
wire						Sln_hwAck;
wire						Sln_fwAck;
wire						Sln_errAck;
wire						Sln_toutSup;
wire						Sln_retry;
wire						Sln_DBusEn;
wire [C_OPB_DWIDTH-1:0]		Sln_DBus;

// Design Under Test
//gfx2d #(C_BASEADDR, C_HIGHADDR, C_OPB_AWIDTH, C_OPB_DWIDTH) ugfx2d (
gfx2d ugfx2d (
	.OPB_ABus(OPB_ABus),
	.OPB_BE(OPB_BE),
	.OPB_Clk(OPB_Clk),
	.OPB_DBus(OPB_DBus),
	.OPB_RNW(OPB_RNW),
	.OPB_Rst(OPB_Rst),
	.OPB_select(OPB_select),
	.OPB_seqAddr(OPB_seqAddr),
	.OPB_hwXfer(OPB_hwXfer), 
	.OPB_fwXfer(OPB_fwXfer), 
	.Sln_xferAck(Sln_xferAck), 
	.Sln_hwAck(Sln_hwAck), 
	.Sln_fwAck(Sln_fwAck), 
	.Sln_errAck(Sln_errAck), 
	.Sln_toutSup(Sln_toutSup), 
	.Sln_retry(Sln_retry), 
	.Sln_DBusEn(Sln_DBusEn), 
	.Sln_DBus(Sln_DBus)
);

// Clock gen
always #10 OPB_Clk = ~OPB_Clk;

// Stimulate inputs
initial begin
	OPB_Clk = 0;
	OPB_Rst = 0;
	OPB_ABus = 0;
	OPB_DBus = 0;
	OPB_BE = 0;
	OPB_RNW = 0;
	OPB_select = 0;
	OPB_seqAddr = 0;
	OPB_hwXfer = 0;
	OPB_fwXfer = 0;

	reset;

	// Test 4-bit status register
	regWrite(0, 32'hFFFF_0003, 4'hf);
	regRead(0);

	// Test data registers
	regWrite(4, 32'h1234_5678, 4'hf);
	regRead(4);

	regWrite(8, 32'hdead_beef, 4'hf);
	regRead(8);

	regWrite(12, 32'h8765_4321, 4'hf);
	regRead(12);

	regWrite(16, 32'hbeef_dead, 4'hf);
	regRead(16);

	// Test non-existant register
	regWrite(20, 32'hbeef_dead, 4'hf);
	regRead(20);

	// Test BE
	regWrite(4, 32'h1234_5678, 4'b1010);
	regRead(4);

	repeat(5) @(posedge OPB_Clk);
	$stop;
end

initial #1000 $stop;

// Asserts reset for 2 clocks then deasserts
task reset;
	begin
		OPB_Rst = 1;
		repeat(2) @(posedge OPB_Clk);
		OPB_Rst = 0;
	end
endtask

task regRead;
	input [C_OPB_AWIDTH-1:0] offset;
	begin
		#1 OPB_ABus = C_BASEADDR + offset;
		OPB_RNW = 1; //read
		OPB_hwXfer = 1;
		OPB_fwXfer = 1;
		OPB_select = 1;
		@(posedge OPB_Clk);
		#1 OPB_ABus = 0;
		OPB_DBus = 0;
		OPB_RNW = 0;
		OPB_hwXfer = 0;
		OPB_fwXfer = 0;
		OPB_select = 0;
		@(negedge Sln_xferAck);
	end
endtask

task regWrite;
	input [C_OPB_AWIDTH-1:0] offset;
	input [C_OPB_DWIDTH-1:0] data;
	input [C_OPB_DWIDTH/8-1:0] be;
	begin
		#1 OPB_ABus = C_BASEADDR + offset;
		OPB_DBus = data;
		OPB_BE = be;
		OPB_RNW = 0; //write
		OPB_hwXfer = 1;
		OPB_fwXfer = 1;
		OPB_select = 1;
		@(posedge OPB_Clk);
		#1 OPB_ABus = 0;
		OPB_DBus = 0;
		OPB_BE = 0;
		OPB_RNW = 0;
		OPB_hwXfer = 0;
		OPB_fwXfer = 0;
		OPB_select = 0;
		@(negedge Sln_xferAck);
	end
endtask

/*
task line;
endtask

task blit;
endtask
*/

endmodule
