`timescale 1ns/1ps

module line_tb ();

reg				OPB_Clk;
reg				OPB_Rst;
reg 			req_line;
reg	[9:0]		x0, x1, y0, y1;
reg [23:0]		RGB;
wire			line_rtr;

wire [159:0]	input_data = {22'b0, y1, 22'b0, x1, 22'b0, y0, 22'b0, x0, 8'b0, RGB};

reg				fifo_full;
wire			fifo_we;
//wire [43:0]		fifo_wd;

wire [9:0]		x_out, y_out;
wire [23:0]		RGB_out;

// Design Under Test
Line uLine(
	.iCLK(OPB_Clk),
	.iRST(OPB_Rst),
	.iSOFT_RST(1'b0),

	// gfx2D / CP interface
	.iREQ(req_line),
	.iDATA(input_data),
	.oRTR(line_rtr),

	// output FIFO interface
	.iFIFO_FULL (fifo_full),
	.oFIFO_WE (fifo_we),
	//.oFIFO_WD (fifo_wd)
	.oFIFO_WD ({x_out, y_out, RGB_out})
);


// Clock gen
always #10 OPB_Clk = ~OPB_Clk;

// Stimulate inputs
initial begin
	OPB_Clk = 0;
	OPB_Rst = 0;

	x0 = 0;
	x1 = 0;
	y0 = 0;
	y1 = 0;
	RGB = 0;
	req_line = 0;
	fifo_full = 0;

	reset;

	/*
	// Case 1: not steep, x1 > x0
	$display("\n\nCase 1\n");
	doLine(0, 0, 10, 2, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);

	// Case 2: not steep, x0 > x1
	$display("\n\nCase 2\n");
	doLine(10, 2, 0, 0, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);

	// Case 3: steep, x1 > x0
	$display("\n\nCase 3\n");
	doLine(0, 0, 2, 10, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);

	// Case 4: steep, x0 > x1
	$display("\n\nCase 4\n");
	doLine(2, 10, 0, 0, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);

	// Case 5
	$display("\n\nCase 5\n");
	doLine(2, 10, 10, 2, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);

	// Case 6
	$display("\n\nCase 6\n");
	doLine(10, 2, 2, 10, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);
	*/

	// Case 7: horizontal
	$display("\n\nCase 7\n");
	doLine(0, 0, 10, 0, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);

	// Case 8: almost horizontal
	$display("\n\nCase 7\n");
	doLine(0, 0, 20, 1, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);

	// Case 9: vertical
	$display("\n\nCase 9\n");
	doLine(0, 0, 0, 10, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);

	// Case 10: almost vertical
	$display("\n\nCase 10\n");
	doLine(0, 0, 1, 20, 24'h00_FFFF);
	repeat(5) @(posedge OPB_Clk);

	//reset;
	//doLine(500, 200, 510, 205, 24'hFF_0000);
	//repeat(5) @(posedge OPB_Clk);
	
	//HN: test with fifo_full = 1

	$stop;
end

initial #5000 $stop; // prevent hang

// Asserts reset for 2 clocks then deasserts
task reset;
	begin
		#3 OPB_Rst = 1;
		repeat(2) @(posedge OPB_Clk);
		#3 OPB_Rst = 0;
	end
endtask

task doLine;
	input [9:0] x0_in;
	input [9:0] y0_in;
	input [9:0] x1_in;
	input [9:0] y1_in;
	input [23:0] RGB_in;
	begin
		#3 x0 = x0_in;
		y0 = y0_in;
		x1 = x1_in;
		y1 = y1_in;
		RGB = RGB_in;
		req_line = 1;
		@(posedge OPB_Clk);
		#3 req_line = 0;
		//@(posedge line_rtr);
		while (!line_rtr) begin
			if (fifo_we) begin
				$display("Fifo out: %d, %d, %x\n", x_out, y_out, RGB_out);
			end
			@(posedge OPB_Clk);
		end
	end
endtask

endmodule
